var fs = require("fs");

var package = JSON.parse(fs.readFileSync("./package.json"));
var debug = process.env.ENVIRONMENT !== "production";

var paths = require("./paths.json"),
  gulp = require("gulp"),
  sass = require("gulp-sass"),
  cssmin = require("gulp-cssmin"),
  plumber = require("gulp-plumber"),
  webserver = require("gulp-webserver"),
  sourcemaps = require("gulp-sourcemaps"),
  templateCache = require("gulp-angular-templatecache"),
  htmlmin = require("gulp-htmlmin"),
  documentation = require("gulp-documentation"),
  autoprefixer = require("gulp-autoprefixer"),
  del = require("del"),
  rename = require("gulp-rename"),
  runSequence = require("run-sequence"),
  webpack = require("webpack"),
  gulpWebpack = require("webpack-stream");

var PLUGINS = [
  new webpack.DefinePlugin({
    ENVIRONMENT: JSON.stringify(process.env.ENVIRONMENT),
    APP: {
      version: JSON.stringify(package.version),
      name: JSON.stringify(package.name),
      author: JSON.stringify(package.author)
    }
  })
];

if (!debug) {
  PLUGINS.push(
    new webpack.optimize.UglifyJsPlugin({ mangle: true, sourcemap: true })
  );
}

/******** TASKS ********/
gulp.task("docs", function() {
  gulp
    .src("src/**/*.js")
    .pipe(documentation({ format: "html" }))
    .pipe(gulp.dest("docs"));
});

// Scripts *******************
gulp.task("scripts", function() {
  gulp
    .src("src/app.js")
    .pipe(
      gulpWebpack(
        {
          devtool: debug ? "inline-sourcemap" : "",
          output: {
            filename: "ingaia-chat.min.js"
          },
          resolve: {
            extensions: [".js"]
          },
          module: {
            rules: [
              {
                test: /\.js$/,
                exclude: /(node_modules|bower_components|test)/,
                loader: "babel-loader",
                query: {
                  presets: ["es2015", "stage-0"],
                  plugins: ["transform-object-rest-spread"]
                }
              },
              {
                test: /\.scss$|.sass$/,
                loader: "style-loader!css-loader!sass-loader"
              },
              {
                test: /\.woff(2)?(\?v=[0-9]\.[0-9]\.[0-9])?$/,
                loader: "url-loader",
                options: {
                  name: "fonts/[name].[ext]",
                  limit: 10000,
                  mimetype: "application/font-woff"
                }
              },
              {
                test: /\.(ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
                loader: "file-loader",
                options: {
                  name: "fonts/[name].[ext]"
                }
              },
              {
                test: /\.(jpe?g|png|gif|svg|jpg)$/i,
                loader: "file-loader",
                options: {
                  name: "imgs/[name].[ext]"
                }
              }
            ]
          },
          plugins: PLUGINS
        },
        webpack
      )
    )
    .pipe(gulp.dest(paths.dist.scripts));
});

// Styles *******************
gulp.task("styles", function() {
  return gulp
    .src(paths.styles_entry)
    .pipe(sass.sync().on("error", sass.logError))
    .pipe(sourcemaps.init())
    .pipe(autoprefixer())
    .pipe(cssmin())
    .pipe(rename("ingaia-chat.min.css"))
    .pipe(sourcemaps.write("."))
    .pipe(gulp.dest(paths.dist.styles));
});

// Fonts *******************
gulp.task("fonts", function() {
  gulp
    .src(paths.third_party.fonts)
    .pipe(gulp.dest(paths.dist.styles + "/fonts"));
});

// Audios *******************
gulp.task("audios", function() {
  gulp.src(paths.third_party.audios).pipe(gulp.dest(paths.dist.audios));
});

// Images *******************
gulp.task("images", function() {
  gulp.src(paths.third_party.images).pipe(gulp.dest(paths.dist.images));
});

gulp.task("assets", ["images", "fonts", "audios"]);

// Template Cache *******************
gulp.task("templatecache", function() {
  console.log("Creating an AngularJS $templateCache");
  return gulp
    .src(paths.app.templates)
    .pipe(htmlmin({ collapseWhitespace: true }))
    .pipe(
      templateCache("templates.js", {
        module: "inGaiaChat",
        root: "ingaia-chat/"
      })
    )
    .pipe(gulp.dest(paths.app.root + "/components/templates"));
});

// Static server *******************
gulp.task("webserver", function() {
  gulp.src("./").pipe(
    webserver({
      port: 3000,
      livereload: true,
      open: false
    })
  );
});

// Demo *******************
gulp.task("demo", function() {
  gulp
    .src("./demo")
    .pipe(
      gulpWebpack({
        devtool: debug ? "inline-sourcemap" : null,
        output: {
          filename: "app.min.js"
        },
        resolve: {
          extensions: [".js"]
        },
        module: {
          loaders: [
            {
              test: /\.js$/,
              exclude: /(node_modules|bower_components)/,
              loader: "babel-loader",
              query: {
                presets: ["es2015", "stage-0"]
              }
            }
          ]
        },
        plugins: debug
      })
    )
    .pipe(gulp.dest(paths.demo.scripts));
});

gulp.task("build-clean", function() {
  return del(paths.dist.root);
});

gulp.task("build", function(callback) {
  runSequence(
    "build-clean",
    ["templatecache", "scripts", "styles", "assets"],
    callback
  );
});

gulp.task("watch", function() {
  gulp.watch(["src/**/*.js"], ["scripts"]);
  gulp.watch(["src/**/*.sass", "src/**/*.scss"], ["styles"]);
  gulp.watch(paths.assets.images + "/*", ["images"]);
  gulp.watch(paths.app.templates, ["templatecache"]);
  gulp.watch(paths.demo.root + "/**/*.js", ["demo"]);
});

gulp.task("dev", function(callback) {
  debug = true;
  runSequence("build", ["webserver", "watch"], callback);
});

gulp.task("staging", function(callback) {
  debug = true;
  runSequence("build", callback);
});

gulp.task("prod", function() {
  debug = false;
  gulp.start("build");
});

gulp.task("default", ["build", "docs"]);

module.exports = {
  env: debug
};
