var appConfig = require("../../config");
var chatApiFactory = function(
  $http,
  $q,
  $log,
  ChatCookies,
  ChatMessageLog,
  ChatStorage,
  ChatUtils
) {
  var defaults = {
    timeout: 60000,
    headers: {
      "Content-Type": "application/json",
      Authorization: "Bearer " + getAPIToken()
    },
    transport: ["webSockets", "foreverFrame", "serverSentEvents"]
  };

  /**
   * Get API Token
   * @method
   * @return {String}
   */
  function getAPIToken() {
    return ChatCookies.read("webapi_token");
  }

  /**
   * Get GaiaCore Token
   * @method
   * @return {String}
   */
  function getGaiaCoreToken() {
    return ChatCookies.read("gaiacore_token");
  }

  /**
   * Get GaiaCore Token
   * @method
   * @return {String}
   */
  function getGaiaCoreUrl() {
    return decodeURIComponent(ChatCookies.read("gaiacore_url"));
  }

  /**
   * Get API URL
   * @method
   * @return {String}
   */
  function getAPIURL() {
    return decodeURIComponent(ChatCookies.read("chat_api_url"));
  }

  /**
   * Get SignalR URL
   * @method
   * @return {String}
   */
  function getSignalRURL() {
    return decodeURIComponent(ChatCookies.read("signalr_url"));
  }

  /**
   * Get Contacts API URL
   * @method
   * @return {String}
   */
  function getContactsURL() {
    return decodeURIComponent(ChatCookies.read("contacts_api_url"));
  }

  /**
   * Returns Hub connection instance
   * @method
   * @return {Object}
   */
  var getProxy = function() {
    if ($.connection) {
      $.connection.hub.url = getSignalRURL() + "/hubs";
      return $.connection.ingaiachat;
    } else {
      return null;
    }
  };

  /**
   * Connect to Chat Hub
   * @method
   * @return {Promise}
   */
  var connectHub = function(transp) {
    transp = transp || defaults.transport;
    $.connection.hub.qs = { token: getAPIToken };
    return $.connection.hub.start({ transport: transp });
  };

  /**
   * Start chat connection with Hub and API
   * @method
   * @return {Promise}
   */
  var startConnection = function(data) {
    var self = this;
    var deferred = $q.defer();
    var hubs = getSignalRURL() + "/hubs";

    var onError = function(err) {
      deferred.reject(err);
      console.error(err);
    };

    try {
      loadScript(hubs).then(
        function() {
          if (data === undefined) {
            onError(ChatMessageLog.ERROR.CONNECTION_PARAMS);
            return deferred.promise;
          }

          self.proxy().client.void = function() {
            /* SignalR needs to create any method on client side to start event listeners */
          };

          // Connect to Hub
          connectHub()
            .done(function() {
              // Connect to API
              connectApi(data.user).then(function(response) {
                // Connection success
                removeLostConnection();
                deferred.resolve(response);
              }, onError);
            })
            .fail(onError);

          // Check if user reconnected using longPolling. If it's, then stop connection to try again with another transport
          self.proxy().on("changeTransport", function(transport) {
            if (
              $.connection.hub.transport &&
              $.connection.hub.transport.name == "longPolling"
            ) {
              $.connection.hub.stop();
            }
          });

          // Notify on disconnections
          $.connection.hub.disconnected(function(err) {
            if (
              $.connection.hub.state !== $.signalR.connectionState.connecting
            ) {
              window.inGaiaChat_API.emit("chatDisconnected");
            }
            if ($.connection.hub.lastError) {
              console.error(
                "Disconnected. Reason: " + $.connection.hub.lastError.message
              );
            }
          });

          // Call disconnected event when occurs a server error (sometimes when Load Balance chane machines)
          $.connection.hub.error(function(err) {
            var currentConnectionId = $.connection.hub.id;
            addLostConnection(currentConnectionId);

            window.inGaiaChat_API.emit("chatDisconnected", err);
          });
        },
        function(err) {
          onError(ChatMessageLog.ERROR.HUBS);
        }
      );
      return deferred.promise;
    } catch (e) {
      return deferred.promise;
    }
  };

  /**
   * Connect to Chat API
   * @method
   * @return {Promise}
   */
  var connectApi = function(user) {
    var deferred = $q.defer();
    getProxy()
      .server.connect(user)
      .done(function(response) {
        deferred.resolve(response);
      })
      .fail(function(err) {
        $log.error("Erro ao connectar chat: ", err);
        deferred.reject(err);
      });
    return deferred.promise;
  };

  /**
   * Add lost connections to localStorage variable last_connections
   * @method
   */
  var addLostConnection = function(connectionId) {
    if (connectionId) {
      ChatStorage.get("last_connections", function(connection) {
        var today = new Date().toLocaleDateString();

        if (connection.date != today) {
          ChatStorage.set({
            last_connections: {
              date: today,
              list: [connectionId]
            }
          });
        } else {
          var list = connection.list || [];
          // Check if current connection is not in last_connections, add it
          if (list.indexOf(connectionId) == -1) {
            list.push(connectionId);

            ChatStorage.set({
              last_connections: {
                date: today,
                list: list
              }
            });
          }
        }
      });
    }
  };

  /**
   * Check for last connection id to send a request and delete it
   * @method
   */
  var removeLostConnection = function() {
    ChatStorage.get("last_connections", function(connection) {
      try {
        var today = new Date().toLocaleDateString();

        if (connection.date != today) return;

        if (connection.list.length) {
          dropConnections(connection.list).then(
            function(response) {
              var remaining = ChatUtils.filterList(
                response.data,
                connection.list
              );
              ChatStorage.set({
                last_connections: {
                  date: connection.date,
                  list: remaining
                }
              });
            },
            function(err) {
              console.error(err.statusText);
            }
          );
        }
      } catch (err) {
        console.error(err);
      }
    });
  };

  /**
   * Drop lost connections
   * @method
   * @return {Promise}
   */
  var dropConnections = function(connections) {
    return $http({
      url: getAPIURL() + "/conn",
      method: "DELETE",
      timeout: defaults.timeout,
      data: connections,
      headers: defaults.headers
    });
  };

  /**
   * Get Room messages
   * @method
   * @return {Array}
   */
  var getMessages = function(roomId, params) {
    return $http({
      url: getAPIURL() + "/message/" + roomId,
      method: "GET",
      timeout: defaults.timeout,
      params: {
        limit: params.limit,
        offset: params.offset,
        hasArchives: params.hasArchives,
        userId: params.userId,
        productTag: params.productTag
      },
      headers: defaults.headers
    });
  };

  /**
   * Get user's conversations
   * @method
   * @return {Array}
   */
  var getConversations = function(params) {
    return $http({
      url: getAPIURL() + "/conversations/" + params.id,
      method: "GET",
      timeout: defaults.timeout,
      params: {
        limit: params.limit,
        offset: params.offset
      },
      isArray: false,
      headers: defaults.headers
    });
  };

  /**
   * Search for conversations by name
   * @method
   * @return {Array}
   */
  var searchConversation = function(params) {
    return $http({
      url: getAPIURL() + "/conversations/getuserbyname/" + params.id,
      method: "GET",
      timeout: defaults.timeout,
      params: {
        name: params.name,
        limit: params.limit,
        offset: params.offset
      },
      headers: defaults.headers
    });
  };

  /**
   * Get user's contact list by endpoint
   * @method
   * @return {Array}
   */
  var getContacts = function(url, params) {
    return $http({
      url: url,
      method: "GET",
      timeout: defaults.timeout,
      params: {
        limit: params.limit,
        offset: params.offset,
        networkId: params.networkId
      },
      isArray: false,
      headers: defaults.headers
    });
  };

  /**
   * Get user's contact list from network
   * @method
   * @return {Array}
   */
  var getNetworkContacts = function(url, params) {
    return $http({
      url: url + "/network/" + params.networkId,
      method: "GET",
      timeout: defaults.timeout,
      params: {
        limit: params.limit,
        offset: params.offset
      },
      isArray: false,
      headers: defaults.headers
    });
  };

  /**
   * Search for contacts by name
   * @method
   * @return {Array}
   */
  var searchContact = function(params) {
    return $http({
      url: getContactsURL() + "/imob/fullSearch",
      method: "POST",
      timeout: defaults.timeout,
      data: {
        name: params.name,
        networkIds: params.networkIds,
        companyId: params.companyId,
        userId: params.userId,
        productTag: params.productTag
      },
      headers: defaults.headers
    });
  };

  /**
   * Add contact to the blocked list
   * @method
   */
  var blockContact = function(data) {
    return $http({
      url: getAPIURL() + "/block",
      method: "POST",
      data: data,
      headers: defaults.headers
    });
  };

  /**
   * Remove contact from the blocked list
   * @method
   */
  var unblockContact = function(data) {
    return $http({
      url: getAPIURL() + "/block",
      method: "DELETE",
      data: data,
      headers: defaults.headers
    });
  };

  /**
   * Get a list of blocked contacts
   * @method
   * @return {Array}
   */
  var getBlockedList = function(params) {
    return $http({
      url: getAPIURL() + "/block",
      method: "GET",
      params: {
        id: params.id,
        productTag: params.productTag
      },
      headers: defaults.headers
    });
  };

  /**
   * Load user's preferences
   * @method
   * @return {Array}
   */
  var loadSettings = function(params) {
    return $http({
      url: getAPIURL() + "/settings/getusersetting/" + params.userId,
      method: "GET",
      params: {
        productTag: params.productTag
      },
      timeout: defaults.timeout,
      headers: defaults.headers
    });
  };

  /**
   * Save user's preferences
   * @method
   */
  var saveSettings = function(userId, productTag, data) {
    return $http({
      url: getAPIURL() + "/settings/createsetting",
      method: "POST",
      headers: defaults.headers,
      params: {
        userId: userId,
        productTag: productTag
      },
      data: data
    });
  };

  /**
   * Get the total number of notifications (messages unread)
   * @method
   * @return {Number}
   */
  var getTotalNotifications = function(params) {
    return $http({
      url: getAPIURL() + "/message/totalUnread/",
      method: "GET",
      timeout: defaults.timeout,
      params: {
        userId: params.userId,
        productTag: params.productTag
      },
      isArray: false,
      headers: defaults.headers
    });
  };

  /**
   * Load a script dynamically
   * @method
   * @return {Promise}
   */
  var loadScript = function(str) {
    var deferred = $q.defer();

    var pendingCount = str.length;
    var url = str;
    var script = document.createElement("script");
    script.type = "text/javascript";

    function stateChange() {
      pendingCount--;
      deferred.resolve();
    }

    script.onerror = function(err) {
      deferred.reject(err);
    };

    if (script.readyState) {
      //IE
      script.onreadystatechange = function() {
        if (script.readyState == "loaded" || script.readyState == "complete") {
          script.onreadystatechange = null;
          stateChange();
        }
      };
      document.write('<script src="' + url + '" defer></' + "script>");
    } else {
      //Others
      script.onload = function() {
        stateChange();
      };
      script.src = url;
      document.getElementsByTagName("head")[0].appendChild(script);
    }

    return deferred.promise;
  };

  // REPORTS
  var sendReportAccess = function() {
    return $http({
      url: getAPIURL() + "/kpi/chatreportview",
      method: "POST",
      timeout: defaults.timeout,
      headers: defaults.headers
    });
  };

  var upload = {
    save: function(file, params) {
      var fd = new FormData();
      fd.append("file", file);
      return $http.post(appConfig.galleryApi, fd, {
        params: {
          roomId: params.roomId,
          imageHeight: params.height,
          imageWidth: params.width
        },
        timeout: defaults.timeout,
        transformRequest: angular.identity,
        headers: {
          "Content-Type": undefined,
          Authorization: "Bearer " + getAPIToken()
        }
      });
    },
    getFileInfo: function(fileId, params) {
      return $http.get(appConfig.galleryApi + fileId, {
        timeout: defaults.timeout,
        headers: defaults.headers
      });
    },
    getImage: function(fileId, params) {
      return $http.get(appConfig.galleryApi + fileId + "/thumbnail", {
        params: {
          requestedWidth: params ? params.width : 200,
          requestedHeight: params ? params.height : 200
        },
        timeout: defaults.timeout,
        headers: defaults.headers
      });
    },
    getFile: function(fileId) {
      return $http.get(appConfig.galleryApi + fileId + "/source", {
        timeout: defaults.timeout,
        headers: defaults.headers
      });
    },
    getUrl: function(fileId) {
      return $http.get(appConfig.galleryApi + fileId + "/url", {
        timeout: defaults.timeout,
        headers: defaults.headers
      });
    },
    download: function(fileId) {
      return $http.get(appConfig.galleryApi + fileId + "/download", {
        timeout: defaults.timeout,
        headers: defaults.headers
      });
    }
  };

  return {
    proxy: getProxy,
    getAPIToken: getAPIToken,
    getGaiaCoreToken: getGaiaCoreToken,
    getGaiaCoreUrl: getGaiaCoreUrl,
    getMessages: getMessages,
    getConversations: getConversations,
    getContacts: getContacts,
    getNetworkContacts: getNetworkContacts,
    getTotalNotifications: getTotalNotifications,
    searchConversation: searchConversation,
    searchContact: searchContact,
    loadSettings: loadSettings,
    saveSettings: saveSettings,
    blockContact: blockContact,
    unblockContact: unblockContact,
    blockedList: getBlockedList,
    removeLostConnection: removeLostConnection,
    start: startConnection,
    connectHub: connectHub,
    connectApi: connectApi,
    sendReportAccess: sendReportAccess,
    upload: upload
  };
};

chatApiFactory.$inject = [
  "$http",
  "$q",
  "$log",
  "ChatCookies",
  "ChatMessageLog",
  "ChatStorage",
  "ChatUtils"
];

var factoriesModules = require("../factories.module");
factoriesModules.factory("ChatAPI", chatApiFactory);
