var factoriesModules = require("../factories.module");
factoriesModules.factory("ChatContacts", [
  "$q",
  "ChatUser",
  "ChatAPI",
  "ChatUtils",
  function($q, ChatUser, ChatAPI, ChatUtils) {
    var defaults = {
      limit: 20,
      offset: 0
    };

    var mergeBlockedWithContacts = function() {
      var self = ContactsFactory;
      ChatUtils.mergeBlockeds(self.list, self.blocked);
    };

    var getContactsFromNetwork = function(url, options, onsuccess, onerror) {
      var self = ContactsFactory;
      var defaultOptions = {
        limit: defaults.limit,
        offset: defaults.offset
      };

      if (options != null || options != undefined) {
        angular.extend(defaultOptions, options);
      }

      self.pending = true;

      ChatAPI.getNetworkContacts(url, defaultOptions)
        .then(
          function(response) {
            var contacts = response.data.contacts;
            if (onsuccess) {
              onsuccess(contacts);
            } else {
              self.list = contacts;
            }
          },
          function(err) {
            if (onerror) {
              onerror(err);
            }
          }
        )
        .finally(function() {
          self.pending = false;
        });
    };

    /**
     * Return an Array of contacts from external API.
     * @method
     * @return {Array}
     */
    var getContacts = function(url, options, onsuccess, onerror) {
      if (!url) return false;

      var self = ContactsFactory;
      var defaultOptions;
      var User = ChatUser.get();

      defaultOptions = {
        companyId: User.company.id,
        limit: defaults.limit,
        offset: defaults.offset
      };

      if (options != null || options != undefined)
        angular.extend(defaultOptions, options);

      self.pending = true;

      ChatAPI.getContacts(url, defaultOptions)
        .then(
          function(response) {
            self.count += response.data.count;
            var contacts = response.data.contacts;

            if (onsuccess) {
              onsuccess(contacts);
            } else {
              self.list = contacts;
            }

            self.mergeBlockedWithContacts();
          },
          function(err) {
            if (onerror) {
              onerror(err);
            }
            console.error("Error getting contacts");
          }
        )
        .finally(function() {
          self.pending = false;
        });
    };

    /**
     * Search for conversations
     * @method
     * @return {Array}
     */
    var search = function(name) {
      var self = ContactsFactory;
      var User = ChatUser.get();
      var deferred = $q.defer();

      var params = {
        userId: User.id,
        productTag: User.productTag,
        companyId: User.company.id,
        networkIds: ChatUtils.getNetworkIds(User.companyFilters),
        name: name
      };

      self.pending = true;

      ChatAPI.searchContact(params)
        .then(
          function(response) {
            var mergedContacts = ChatUtils.mergeBlockeds(
              response.data.contacts,
              self.blocked
            );
            deferred.resolve(mergedContacts);
          },
          function(err) {
            deferred.reject(err);
          }
        )
        .finally(function() {
          self.pending = false;
        });

      return deferred.promise;
    };

    /**
     * Return an Array of blocked contacts.
     * @method
     * @params {id: int, productTag: string}
     * @return {Array}
     */
    var getBlockedList = function(onsuccess, onerror) {
      var self = ContactsFactory;
      var User = ChatUser.get();

      ChatAPI.blockedList(User).then(
        function(response) {
          self.blocked = response.data;
          self.mergeBlockedWithContacts();
          if (onsuccess != undefined) {
            onsuccess(response.data);
          }
        },
        function(err) {
          if (onerror != undefined) {
            onerror(err);
          }
          console.error(err.statusText);
        }
      );
    };

    /**
     * Unblocked a contact.
     * @method
     * @params {id: int, productTag: string}
     * @return {Array}
     */
    var unblock = function(blocked, onsuccess, onerror) {
      var self = ContactsFactory;
      var User = ChatUser.get();
      var params = [User, blocked];

      ChatAPI.unblockContact(params).then(
        function() {
          var list = self.blocked.filter(function(contact) {
            return blocked.id != contact.id;
          });

          self.blocked = list;

          if (onsuccess != undefined) {
            onsuccess(self.blocked);
          }

          window.inGaiaChat_API.emit("updateBlockedList");
          window.inGaiaChat_API.emit("updateConversationsList");
          window.inGaiaChat_API.emit("chatContactBlockChange", blocked);
        },
        function(err) {
          if (onerror) {
            onerror(err);
          }
          console.error(err);
        }
      );
    };

    /**
     * Block a contact.
     * @method
     * @params {id: int, productTag: string}
     * @return {Array}
     */
    var block = function(blocked, onsuccess, onerror) {
      var self = ContactsFactory;
      var User = ChatUser.get();
      var params = [User, blocked];

      ChatAPI.blockContact(params).then(
        function() {
          self.blocked.push(blocked);
          self.mergeBlockedWithContacts();

          if (onsuccess != undefined) {
            onsuccess(self.blocked);
          }

          window.inGaiaChat_API.emit("updateBlockedList");
          window.inGaiaChat_API.emit("updateConversationsList");
          window.inGaiaChat_API.emit("chatContactBlockChange", blocked);
        },
        function(err) {
          if (onerror) {
            onerror(err);
          }
          console.error(err.status);
        }
      );
    };

    var ContactsFactory = {
      url: "",
      list: [],
      pending: false,
      count: 0,
      blocked: [],
      mergeBlockedWithContacts: mergeBlockedWithContacts,
      getContacts: getContacts,
      getContactsFromNetwork: getContactsFromNetwork,
      getBlockedList: getBlockedList,
      block: block,
      unblock: unblock,
      search: search
    };

    return ContactsFactory;
  }
]);
